@@ -97,13 +97,13 @@ protectFieldText' s =
 
 -- | This may have bad performance issues (dsf: Whoever wrote this
 -- comment should have explained why.)
-instance (ControlFunctions a, Pretty (PP a)) => Pretty (PP (Control' a)) where
-    pPrint = ppControl . unPP
-instance (ControlFunctions a, Pretty (PP a)) => Pretty (PP (Paragraph' a)) where
-    pPrint = ppParagraph . unPP
+instance (ControlFunctions a, Pretty (PP a)) => Pretty (Control' a) where
+    pPrint = ppControl
+instance (ControlFunctions a, Pretty (PP a)) => Pretty (Paragraph' a) where
+    pPrint = ppParagraph
 
-instance (ControlFunctions a, Pretty (PP a)) => Pretty (PP (Field' a)) where
-    pPrint = ppField . unPP
+instance (ControlFunctions a, Pretty (PP a)) => Pretty (Field' a) where
+    pPrint = ppField
 
 ppControl :: (ControlFunctions a, Pretty (PP a)) => Control' a -> Doc
 ppControl (Control paragraph) =
@@ -169,7 +169,7 @@ parseControlFromCmd cmd =
       result <- parseControlFromHandle cmd outh
       either (return . Left . show) (finish handle) result
     where
-      finish handle control = 
+      finish handle control =
           do
             exitCode <- waitForProcess handle
             case exitCode of
@@ -185,7 +185,7 @@ md5sumField :: (ControlFunctions a) => Paragraph' a -> Maybe a
 md5sumField p =
     case fieldValue "MD5Sum" p of
       m@(Just _) -> m
-      Nothing -> 
+      Nothing ->
           case fieldValue "Md5Sum" p of
             m@(Just _) -> m
             Nothing -> fieldValue "MD5sum" p
