@@ -2,13 +2,12 @@ module Main where
 
 import Test.HUnit
 import System.Exit
-import Test.Changes
-import Test.Control
-import Test.Versions
---import Test.VersionPolicy
-import Test.SourcesList
-import Test.Dependencies
-import Text.PrettyPrint.ANSI.Leijen (Doc, text, (<+>), (<$>), fillSep, renderPretty, displayS)
+import Changes
+import Control
+import Versions
+import SourcesList
+import Dependencies
+import Text.PrettyPrint
 
 main =
     do (c,st) <- runTestText putTextToShowS (TestList (versionTests ++ sourcesListTests ++ dependencyTests ++ changesTests ++ controlTests ++ prettyTests))
@@ -27,14 +26,14 @@ prettyTests =
                   "",
                   "Find all the packages referenced by the",
                   "second sources.list which trump packages",
-                  "find in the first sources.list."])
-                (displayS (renderPretty 1.0 40 (helpText "debian-report")) "")
+                  "found in the first sources.list."])
+                (renderStyle (style {lineLength = 60}) (helpText "debian-report"))
                ) ]
 
 helpText :: String -> Doc
 helpText progName =
-    (text "Usage:" <+> text progName <+> text "<old sources.list>" <+> text "<new sources.list>" <$>
-     text [] <$>
-     (fillSep $ map text $ words $ "Find all the packages referenced by the second sources.list which trump packages find in the first sources.list.") <$>
+    (text "Usage:" <+> text progName <+> text "<old sources.list>" <+> text "<new sources.list>" $$
+     text [] $$
+     (fsep $ map text $ words $ "Find all the packages referenced by the second sources.list which trump packages found in the first sources.list.") $$
      text []
     )
