@@ -2,7 +2,6 @@
 module Main where
 
 import Control.Monad
-import Data.List
 import Data.Maybe (fromMaybe)
 import Debian.Apt.Methods
 import Debian.Report
@@ -43,7 +42,7 @@ main =
       mkDocument styleSheet elem =
           let xmlDecl = XMLDecl "1.0" (Just (EncodingDecl "utf-8")) (Just True)
               prolog   = Prolog (Just xmlDecl)  [] Nothing [PI ("xml-stylesheet","type=\"text/xsl\" href=\""++styleSheet++"\"")]
-              symTable = []
+              -- symTable = []
           in
             Document prolog [] elem []
 
@@ -51,11 +50,11 @@ main =
 
 helpText :: String -> Doc
 helpText progName =
-    (text "Usage:" <+> text progName <+> text "<old sources.list>" <+> text "<new sources.list>"$+$ 
-     text [] $+$ 
+    (text "Usage:" <+> text progName <+> text "<old sources.list>" <+> text "<new sources.list>"$+$
+     text [] $+$
      (fsep $ map text $ words $ "Find all the packages referenced by the second sources.list which trump packages find in the first sources.list.")
     )
-    
+
 parseArgs :: IO (String, String)
 parseArgs =
     do args <- getArgs
@@ -71,7 +70,7 @@ parseArgs =
              hPutStrLn stderr =<< renderWidth (helpText progName)
              exitFailure
       -- |render a Doc using the current terminal width
-      renderWidth :: Doc -> IO String       
+      renderWidth :: Doc -> IO String
       renderWidth doc =
           do columns <- return . fromMaybe 80 =<< getWidth
              return $ renderStyle (Style PageMode columns 1.0) doc
